@@ -1,13 +1,13 @@
 ;;; magit-annex.el --- Control git-annex from Magit  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2013-2018 Kyle Meyer <kyle@kyleam.com>
+;; Copyright (C) 2013-2021 Kyle Meyer <kyle@kyleam.com>
 
 ;; Author: Kyle Meyer <kyle@kyleam.com>
 ;;         Rémi Vanicat <vanicat@debian.org>
 ;; URL: https://github.com/magit/magit-annex
 ;; Keywords: vc tools
-;; Version: 1.7.1
-;; Package-Requires: ((cl-lib "0.3") (magit "2.90.0"))
+;; Version: 1.8.0
+;; Package-Requires: ((cl-lib "0.3") (magit "3.0.0"))
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -47,11 +47,11 @@
 ;;    The above commands, which operate on paths, are also useful
 ;;    outside of Magit buffers, especially in Dired buffers.  To make
 ;;    these commands easily accessible in Dired, you can add a binding
-;;    for `magit-annex-file-action-popup'.  If you use git-annex.el,
-;;    you can put the popup under the same binding (@f) with
+;;    for `magit-annex-file-action'.  If you use git-annex.el, you can
+;;    put the popup under the same binding (@f) with
 ;;
 ;;     (define-key git-annex-dired-map "f"
-;;       #'magit-annex-file-action-popup)
+;;       #'magit-annex-file-action)
 ;;
 ;;   @u    Browse unused files.
 ;;   @l    List annex files.
@@ -84,7 +84,7 @@
 
 (require 'cl-lib)
 (require 'magit)
-(require 'magit-popup)
+(require 'transient)
 
 
 ;;; Variables
@@ -146,97 +146,133 @@ program used to open the unused file."
   :type 'boolean)
 
 
-;;; Popups
+;;; Transients
+;;;; Infix Arguments
 
-(magit-define-popup magit-annex-popup
-  "Popup console for git-annex commands."
-  'magit-popups
+(transient-define-argument magit-annex:--jobs ()
+  :description "Number of concurrent jobs"
+  :class 'transient-option
+  :key "-j"
+  :shortarg "-J"
+  :argument "--jobs="
+  :reader 'transient-read-number-N+)
+
+(transient-define-argument magit-annex:--fast ()
+  :description "Fast variant of command"
+  :class 'transient-option
+  :key "-f"
+  :argument "--fast")
+
+(transient-define-argument magit-annex:--force ()
+  :description "Force unsafe actions"
+  :class 'transient-option
+  :key "-F"
+  :argument "--force")
+
+(transient-define-argument magit-annex:--from ()
+  :description "From remote"
+  :class 'transient-option
+  :key "=f"
+  :argument "--from="
+  :reader 'magit-read-remote)
+
+(transient-define-argument magit-annex:--to ()
+  :description "To remote"
+  :class 'transient-option
+  :key "=t"
+  :argument "--to="
+  :reader 'magit-read-remote)
+
+;;;; Prefix commands
+
+;;;###autoload (autoload 'magit-annex-dispatch "magit-annex" nil t)
+(transient-define-prefix magit-annex-dispatch ()
+  "Invoke a git-annex command."
   :man-page "git-annex"
-  :actions  '((?a "Add" magit-annex-add)
-              (?@ "Add" magit-annex-add)
-              (?A "Add all" magit-annex-add-all)
-              (?f "Action on files" magit-annex-file-action-popup)
-              (?G "Get all (auto)" magit-annex-get-all-auto)
-              (?y "Sync" magit-annex-sync-popup)
-              (?m "Merge" magit-annex-merge)
-              (?u "Unused" magit-annex-unused-popup)
-              (?l "List files" magit-annex-list-popup)
-              (?: "Annex subcommand (from pwd)" magit-annex-command)
-              (?! "Running" magit-annex-run-popup))
-  :max-action-columns 3)
+  ["Actions"
+   [("a" "Add" magit-annex-add)
+    ("@" "Add" magit-annex-add)
+    ("A" "Add all" magit-annex-add-all)]
+   [("G" "Get all (auto)" magit-annex-get-all-auto)
+    ("m" "Merge annex branches" magit-annex-merge)
+    (":" "Annex subcommand (from pwd)" magit-annex-command)]]
+  ["Transient commands"
+   [("f" "Action on files" magit-annex-file-action)
+    ("y" "Sync" magit-annex-sync)
+    ("!" "Running" magit-annex-run-command)]
+   [("u" "Unused" magit-annex-unused)
+    ("l" "List files" magit-annex-list)]])
 
-(magit-define-popup magit-annex-file-action-popup
-  "Popup console for git-annex file commands."
-  'magit-annex-popups
+(transient-define-prefix magit-annex-file-action ()
+  "Invoke a git-annex file command."
   :man-page "git-annex"
-  :actions  '((?g "Get" magit-annex-get-files)
-              (?d "Drop" magit-annex-drop-files)
-              (?c "Copy" magit-annex-copy-files)
-              (?m "Move" magit-annex-move-files)
-              (?l "Lock" magit-annex-lock-files)
-              (?u "Unlock" magit-annex-unlock-files) nil nil
-              (?U "Undo" magit-annex-undo-files))
-  :switches '("Switches"
-              (?f "Fast" "--fast")
-              (?F "Force" "--force")
-              "Switches for get, drop, copy, and move"
-              (?a "Auto" "--auto"))
-  :options  '("Options for get, drop, copy, and move"
-              (?t "To remote" "--to=" magit-read-remote)
-              (?f "From remote" "--from=" magit-read-remote)
-              (?n "Number of copies" "--numcopies=")
-              (?j "Number of jobs" "--jobs="))
-  :max-action-columns 4)
+  ["Arguments"
+   (magit-annex:--fast)
+   (magit-annex:--force)]
+  ["Arguments for get, drop, copy, and move"
+   ("-a" "Auto" "--auto")
+   (magit-annex:--from)
+   ("-n" "Desired number of copies" "--numcopies=" transient-read-number-N+)
+   (magit-annex:--jobs)]
+  ["Arguments for copy and move"
+   (magit-annex:--to)]
+  ["Arguments for get, drop, copy, and move"
+   ("=f" "From remote" "--from=" magit-read-remote)]
+  ["Actions"
+   [("g" "Get" magit-annex-get-files)
+    ("d" "Drop" magit-annex-drop-files)
+    ("c" "Copy" magit-annex-copy-files)
+    ("m" "Move" magit-annex-move-files)]
+   [("l" "Lock" magit-annex-lock-files)
+    ("u" "Unlock" magit-annex-unlock-files)
+    ("U" "Undo" magit-annex-undo-files)]])
 
-(magit-define-popup magit-annex-sync-popup
-  "Popup console for git annex sync."
-  'magit-annex-popups
+(transient-define-prefix magit-annex-sync ()
+  "Invoke 'git annex sync'."
   :man-page "git-annex-sync"
-  :actions  '((?y "Sync" magit-annex-sync)
-              (?r "Sync remote" magit-annex-sync-remote))
-  :switches '((?c "Content" "--content")
-              (?f "Fast" "--fast")
-              (?F "Force" "--force")
-              (?n "Don't commit local changes" "--no-commit"))
-  :options  '((?j "Number of jobs" "--jobs="))
-  :default-action 'magit-annex-sync)
+  ["Arguments"
+   (magit-annex:--fast)
+   (magit-annex:--force)
+   ("-c" "Transfer content" "--content")
+   ("-n" "Don't commit local changes" "--no-commit")
+   (magit-annex:--jobs)]
+  ["Actions"
+   ("y" "Sync all remotes" magit-annex-sync-all)
+   ("r" "Sync a remote" magit-annex-sync-remote)])
 
-(magit-define-popup magit-annex-unused-popup
-  "Popup console for git annex unused."
-  'magit-annex-popups
+(transient-define-prefix magit-annex-unused ()
+  "Invoke 'git annex unused'."
   :man-page "git-annex-unused"
-  :actions  '((?u "Unused" magit-annex-unused)
-              (?r "Unused in reflog" magit-annex-unused-reflog))
-  :switches '((?f "Fast" "--fast"))
-  :options '((?f "From remote" "--from=" magit-read-remote)
-             (?r "Refspec" "--used-refspec="))
-  :default-action 'magit-annex-unused)
+  ["Arguments"
+   (magit-annex:--fast)
+   (magit-annex:--from)
+   ("-r" "Refspec" "--used-refspec=")]
+  ["Actions"
+   ("u" "Unused" magit-annex-unused-in-refs)
+   ("r" "Unused in reflog" magit-annex-unused-in-reflog)])
 
-(magit-define-popup magit-annex-list-popup
-  "Popup console for git annex list."
-  'magit-annex-popups
+(transient-define-prefix magit-annex-list ()
+  "Invoke 'git annex list'."
   :man-page "git-annex-list"
-  :actions  '((?l "List files" magit-annex-list-files)
-              (?d "List files in directory" magit-annex-list-dir-files))
-  :switches '((?a "All repos" "--allrepos"))
-  :default-action 'magit-annex-list-files)
+  ["Arguments"
+   ("-a" "All repos" "--allrepos")]
+  ["Actions"
+   ("l" "List files" magit-annex-list-files)
+   ("d" "List files in directory" magit-annex-list-dir-files)])
 
-(magit-define-popup magit-annex-run-popup
-  "Popup console for running git-annex commands."
-  'magit-annex-popups
+(transient-define-prefix magit-annex-run-command ()
+  "Run an arbitrary git-annex command."
   :man-page "git-annex"
-  :actions '((?! "Annex subcommand (from root)" magit-annex-command-topdir)
-             (?: "Annex subcommand (from pwd)" magit-annex-command)))
+  ["Actions"
+   ("!" "Annex subcommand (from root)" magit-annex-command-topdir)
+   (":" "Annex subcommand (from pwd)" magit-annex-command)])
 
 ;;;###autoload
 (eval-after-load 'magit
   '(progn
-     (require 'magit-popup)
-     (when (boundp 'magit-dispatch-popup)
-       (define-key magit-mode-map "@" 'magit-annex-popup-or-init)
-       (magit-define-popup-action 'magit-dispatch-popup
-         ?@ "Annex" 'magit-annex-popup-or-init ?!))))
-
+     (define-key magit-mode-map "@" 'magit-annex-dispatch-or-init)
+     (transient-append-suffix 'magit-dispatch "%"
+       '("@" "Annex" magit-annex-dispatch-or-init))))
 
 
 ;;; Process calls
@@ -257,7 +293,7 @@ information."
   (magit-run-git-async "annex" magit-annex-global-arguments args))
 
 (defun magit-annex-command (command)
-  "Execute COMMAND asynchonously, displaying output.
+  "Execute COMMAND asynchronously, displaying output.
 This is like `magit-git-command', but \"git annex \" rather than
 \"git \" is used as the initial input."
   (interactive (list (magit-read-shell-command nil "git annex ")))
@@ -274,12 +310,12 @@ rather than \"git \" is used as the initial input."
 ;;; Initialization
 
 ;;;###autoload
-(defun magit-annex-popup-or-init ()
-  "Call Magit-annex popup or offer to initialize non-annex repo."
+(defun magit-annex-dispatch-or-init ()
+  "Call `magit-annex-dispatch' or offer to initialize non-annex repo."
   (interactive)
   (cond
    ((magit-annex-inside-annexdir-p)
-    (magit-annex-popup))
+    (magit-annex-dispatch))
    ((y-or-n-p (format "No git-annex repository in %s.  Initialize one? "
                       default-directory))
     (call-interactively 'magit-annex-init))))
@@ -301,6 +337,9 @@ rather than \"git \" is used as the initial input."
   "Add the item at point to annex.
 With a prefix argument, prompt for FILE.
 \('git annex add')"
+  ;; NEEDSWORK: Use of `magit-annex-unlocked-files' doesn't make sense
+  ;; for v6+ repos.
+  ;;
   ;; Modified from `magit-stage'.
   (interactive
    (when current-prefix-arg
@@ -339,7 +378,10 @@ With a prefix argument, prompt for FILE.
 
 ;;; Updating
 
-(defun magit-annex-sync (&optional args)
+(defun magit-annex-sync-arguments ()
+  (transient-args 'magit-annex-sync))
+
+(defun magit-annex-sync-all (&optional args)
   "Sync git-annex.
 \('git annex sync [ARGS]')"
   (interactive (list (magit-annex-sync-arguments)))
@@ -361,6 +403,31 @@ With a prefix argument, prompt for FILE.
 
 ;;; Managing content
 
+(defun magit-annex-files ()
+  "Return all annex files."
+  (magit-git-items "annex" "find" "--print0" "--include" "*"))
+
+(defun magit-annex-present-files ()
+  "Return annex files that are present in current repo."
+  (magit-git-items "annex" "find" "--print0"))
+
+(defun magit-annex-absent-files ()
+  "Return annex files that are absent in current repo."
+  (magit-git-items "annex" "find" "--print0" "--not" "--in=here"))
+
+(defun magit-annex-unlocked-files ()
+  "Return unlocked annex files."
+  (with-temp-buffer
+    (let ((exit (magit-process-file
+                 magit-git-executable nil t nil
+                 "annex" "find" "--print0" "--unlocked")))
+      (if (zerop exit)
+          (split-string (buffer-string) "\0" t)
+        ;; `find --unlocked' isn't available until git-annex
+        ;; 7.20191009.  Fall back to the old approach that is
+        ;; compatible with v5 repos only.
+        (magit-git-items "diff-files" "-z" "--diff-filter=T" "--name-only")))))
+
 (defun magit-annex-get-all-auto ()
   "Run `git annex get --auto'."
   (interactive)
@@ -406,77 +473,96 @@ With a prefix argument, prompt for FILE.
           (dired-relist-file (expand-file-name file)))
       (goto-char here))))
 
-(defmacro magit-annex-files-action (command &optional limit no-async)
-  (declare (indent defun) (debug t))
-  `(defun ,(intern (concat "magit-annex-" command "-files"))
-       (files &optional args)
-     ,(format "%s FILES.\n\n  git annex %s [ARGS] [FILE...]"
-              (capitalize command) command)
-     (interactive
-      (list
-       (let* ((files (or (mapcar #'cdr (magit-region-values 'annex-list-file))
-                         (when-let ((file (cdr (magit-section-value-if
-                                                'annex-list-file))))
-                           (list file))
-                         (and (derived-mode-p 'dired-mode)
-                              (dired-get-marked-files t))))
-              (default (mapconcat #'identity files ",")))
-         (magit-annex-read-files
-          (concat ,(capitalize command)
-                  " file,s"
-                  (and files (format " (%s)" default))
-                  ": ")
-          ,limit
-          default))
-       (magit-annex-file-action-arguments)))
-     (,(if no-async 'magit-annex-run 'magit-annex-run-async)
-      ,command args files)
-     (when (derived-mode-p 'dired-mode)
-       (if ,no-async
-           (magit-annex--dired-relist files)
-         (set-process-sentinel
-          magit-this-process
-          (lambda (process event)
-            (magit-process-sentinel process event)
-            (when (eq (process-status process) 'exit)
-              (magit-annex--dired-relist files)))))
+(defun magit-annex--dired-relist-async (files)
+  (when (derived-mode-p 'dired-mode)
+       (set-process-sentinel
+        magit-this-process
+        (lambda (process event)
+          (magit-process-sentinel process event)
+          (when (eq (process-status process) 'exit)
+            (magit-annex--dired-relist files))))
        (let ((magit-display-buffer-noselect t))
-         (magit-process-buffer)))))
+         (magit-process-buffer))))
 
-(magit-annex-files-action "get" 'absent)
-(magit-annex-files-action "drop"
-  (and (not (magit-annex-from-in-options-p)) 'present))
-(magit-annex-files-action "copy"
-  (and (not (magit-annex-from-in-options-p)) 'present))
-(magit-annex-files-action "move"
-  (and (not (magit-annex-from-in-options-p)) 'present))
-(magit-annex-files-action "unlock" 'present t)
-(magit-annex-files-action "lock" 'unlocked t)
+(defun magit-annex-file-action-arguments ()
+  (transient-args 'magit-annex-file-action))
 
-(magit-annex-files-action "undo" nil t)
+(defun magit-annex--file-arguments (&optional limit-to unless-from)
+  "Return interactive arguments for file-based commands.
+LIMIT-TO is interpreted by `magit-annex-read-files'.  If
+UNLESS-FROM is non-nil, pass LIMIT-TO only if the command
+arguments don't include --from."
+  (let* ((args (magit-annex-file-action-arguments))
+         (files (or (mapcar #'cdr (magit-region-values 'annex-list-file))
+                    (when-let ((file (cdr (magit-section-value-if
+                                           'annex-list-file))))
+                      (list file))
+                    (and (derived-mode-p 'dired-mode)
+                         (dired-get-marked-files t))))
+         (default (mapconcat #'identity files ",")))
+    (list
+     (magit-annex-read-files
+      (concat "File,s"
+              (and files (format " (%s)" default))
+              ": ")
+      (and (or (not unless-from)
+               (cl-notany (lambda (it) (string-match-p "--from=" it))
+                          args))
+           limit-to)
+      default)
+     args)))
 
-(defun magit-annex-from-in-options-p ()
-  (cl-some (lambda (it) (string-match-p "--from=" it))
-           magit-current-popup-args))
+(defun magit-annex-get-files (files &optional args)
+  "Get annex files.
+\('git annex get [ARGS] -- FILES)"
+  (interactive (magit-annex--file-arguments 'absent))
+  (magit-annex-run-async "get" args "--" files)
+  (magit-annex--dired-relist-async files))
 
-(defun magit-annex-files ()
-  "Return all annex files."
-  (magit-git-items "annex" "find" "--print0" "--include" "*"))
+(defun magit-annex-drop-files (files &optional args)
+  "Drop annex files.
+\('git annex drop [ARGS] -- FILES)"
+  (interactive (magit-annex--file-arguments 'present t))
+  (magit-annex-run-async "drop" args "--" files)
+  (magit-annex--dired-relist-async files))
 
-(defun magit-annex-present-files ()
-  "Return annex files that are present in current repo."
-  (magit-git-items "annex" "find" "--print0"))
+(defun magit-annex-copy-files (files &optional args)
+  "Copy annex files.
+\('git annex copy [ARGS] -- FILES)"
+  (interactive (magit-annex--file-arguments 'present t))
+  (magit-annex-run-async "copy" args "--" files)
+  (magit-annex--dired-relist-async files))
 
-(defun magit-annex-absent-files ()
-  "Return annex files that are absent in current repo."
-  (magit-git-items "annex" "find" "--print0" "--not" "--in=here"))
+(defun magit-annex-move-files (files &optional args)
+  "Move annex files.
+\('git annex move [ARGS] -- FILES)"
+  (interactive (magit-annex--file-arguments 'present t))
+  (magit-annex-run-async "move" args "--" files)
+  (magit-annex--dired-relist-async files))
 
-(defun magit-annex-unlocked-files ()
-  "Return unlocked annex files."
-  (magit-git-items "diff-files" "-z" "--diff-filter=T" "--name-only"))
+(defun magit-annex-unlock-files (files &optional args)
+  "Unlock annex files.
+\('git annex unlock [ARGS] -- FILES)"
+  (interactive (magit-annex--file-arguments 'present))
+  (magit-annex-run "unlock" args "--" files)
+  (magit-annex--dired-relist files))
+
+(defun magit-annex-lock-files (files &optional args)
+  "Lock annex files.
+\('git annex lock [ARGS] -- FILES)"
+  (interactive (magit-annex--file-arguments 'unlocked))
+  (magit-annex-run "lock" args "--" files)
+  (magit-annex--dired-relist files))
+
+(defun magit-annex-undo-files (files &optional args)
+  "Undo annex files.
+\('git annex undo [ARGS] -- FILES)"
+  (interactive (magit-annex--file-arguments))
+  (magit-annex-run "undo" args "--" files)
+  (magit-annex--dired-relist files))
 
 
-;; Unused mode
+;;; Unused mode
 
 (defun magit-annex-unused-add ()
   "Add annex unused data back into the index."
@@ -504,31 +590,39 @@ With prefix argument FORCE, pass \"--force\" flag to
                                        '("--force" "all")
                                      "all")))))
 
-(defun magit-annex-unused-log-popup ()
+(defun magit-annex-unused-log ()
   "Display log for unused file.
 
-This is like `magit-log-popup', but, if point is on an unused
-file, the unused file's key is automatically supplied as the
-value for the '-S' flag.  The '--stat' flag is also enabled if
-`magit-annex-unused-stat-argument' is non-nil.
+Show a log where the key for the unused file at point is supplied
+as the value for the '-S' flag.  The '--stat' flag is also
+enabled if `magit-annex-unused-stat-argument' is non-nil.
 
 \('git log [--stat] -S<KEY>')"
   (interactive)
-  (let ((section (magit-current-section)))
-    (if (not (eq (oref section type) 'unused-data))
-        (call-interactively #'magit-log-popup)
-      (let ((magit-log-arguments
-             `(,(concat "-S" (cdr (oref section value)))
-               ,(and magit-annex-unused-stat-argument "--stat")
-               ,@(cl-remove-if
-                  (lambda (x) (string-prefix-p "-S" x))
-                  (-if-let (buffer (magit-mode-get-buffer 'magit-log-mode))
-                      (with-current-buffer buffer
-                        (magit-popup-import-file-args (nth 1 magit-refresh-args)
-                                                      (nth 2 magit-refresh-args)))
-                    (default-value 'magit-log-arguments)))))
-            (magit-popup-use-prefix-argument 'default))
-        (magit-invoke-popup 'magit-log-popup nil nil)))))
+  (magit-section-case
+    (unused-data
+     (let ((args (car (magit-log-arguments))))
+       (when (and magit-annex-unused-stat-argument
+                  (not (member "--stat" args)))
+         (push "--stat" args))
+       (funcall
+        (cond
+         ;; `magit-git-log' was renamed to `magit-log-setup-buffer' in
+         ;; v2.90.1-480-g249ce0eec.  This is a temporary compatibility
+         ;; kludge for Guix, whose current version of Magit is from a
+         ;; bit before that (v2.90.1-460-gc761d28d4).
+         ((fboundp 'magit-log-setup-buffer)
+          #'magit-log-setup-buffer)
+         ((fboundp 'magit-git-log)
+          #'magit-git-log)
+         (t (error "bug: should never get here")))
+        (list (or (magit-get-current-branch) "HEAD"))
+        (cons (concat "-S" (cdr (oref it value)))
+              (cl-remove-if (lambda (x) (string-prefix-p "-S" x))
+                            args))
+        nil)))
+    (t
+     (user-error "No unused file at point"))))
 
 (defun magit-annex--file-name-from-key (key)
   (magit-git-string "annex" "contentlocation" key))
@@ -563,7 +657,7 @@ the file within Emacs."
     (define-key map (kbd "RET") #'magit-annex-unused-open)
     (define-key map "s" #'magit-annex-unused-add)
     (define-key map "k" #'magit-annex-unused-drop)
-    (define-key map "l" #'magit-annex-unused-log-popup)
+    (define-key map (kbd "C-c C-l") #'magit-annex-unused-log)
     map)
   "Keymap for `magit-annex-unused-mode'.")
 
@@ -573,40 +667,51 @@ the file within Emacs."
 \\<magit-annex-unused-mode-map>\
 Type \\[magit-annex-unused-drop] to drop data at point.
 Type \\[magit-annex-unused-add] to add the unused data back into the index.
-Type \\[magit-annex-unused-log-popup] to show commit log for the unused file.
+Type \\[magit-annex-unused-log] to show commit log for the unused file.
 Type \\[magit-annex-unused-open] to open the file.
 \n\\{magit-annex-unused-mode-map}"
   :group 'magit-modes
   (hack-dir-local-variables-non-file-buffer))
 
-;;;###autoload
-(defun magit-annex-unused (&optional args)
-  "Show unused data.
-\('git annex unused [ARGS]')"
-  (interactive (list (magit-annex-unused-arguments)))
-  (magit-mode-setup #'magit-annex-unused-mode args))
+(defun magit-annex-unused-arguments ()
+  (transient-args 'magit-annex-unused))
 
 ;;;###autoload
-(defun magit-annex-unused-reflog (&optional args)
-  "Show unused data.
+(defun magit-annex-unused-in-refs (&optional args)
+  "Show annex files not used in any branches or tags.
+These files are not pointed by the tips of the repositories
+branches or tags.
+\('git annex unused [ARGS]')"
+  (interactive (list (magit-annex-unused-arguments)))
+  (magit-annex-unused-setup-buffer args))
+
+;;;###autoload
+(defun magit-annex-unused-in-reflog (&optional args)
+  "Show annex files not used in any of the revisions in HEAD's reflog.
 \('git annex unused --used-refspec=reflog [ARGS]')"
   (interactive (list (magit-annex-unused-arguments)))
   (if (cl-some (lambda (x) (string-prefix-p "--used-refspec=" x))
                args)
       (user-error "Flag --used-refspec was given more than once")
     (setq args (cons "--used-refspec=reflog" args)))
-  (magit-mode-setup #'magit-annex-unused-mode args))
+  (magit-annex-unused-setup-buffer args))
 
-(defun magit-annex-unused-refresh-buffer (args)
+(defun magit-annex-unused-setup-buffer (args)
+  (magit-setup-buffer #'magit-annex-unused-mode nil
+    (magit-buffer-arguments args)))
+
+(defun magit-annex-unused-refresh-buffer ()
   "Refresh the content of the unused buffer."
   (magit-insert-section (unused)
     (magit-insert-heading
       (concat "Unused files"
-              (and args
-                   (concat " (" (mapconcat #'identity args " ") ")"))
+              (and magit-buffer-arguments
+                   (concat " ("
+                           (mapconcat #'identity magit-buffer-arguments " ")
+                           ")"))
               ":"))
     (magit-git-wash #'magit-annex-unused-wash
-      "annex" "unused" args)))
+      "annex" "unused" magit-buffer-arguments)))
 
 (defun magit-annex-unused-wash (&rest _)
   "Convert the output of git-annex unused into Magit section."
@@ -647,26 +752,31 @@ Type \\[magit-annex-unused-open] to open the file.
 (defvar magit-annex-list-mode-map
   (let ((map (make-sparse-keymap)))
     (set-keymap-parent map magit-mode-map)
-    (define-key map "f" #'magit-annex-file-action-popup)
+    (define-key map "f" #'magit-annex-file-action)
     map)
   "Keymap for `magit-annex-list-mode'.")
 
+(defvar-local magit-annex-buffer-directory nil)
+
 (define-derived-mode magit-annex-list-mode magit-mode "Magit-annex List"
   "Mode for viewing on `git annex list' output.
 
 \\<magit-annex-list-mode-map>\
-Type \\[magit-annex-file-action-popup] to perform git-annex action
+Type \\[magit-annex-file-action] to perform git-annex action
 on the files selected by the region (if active) or the file at point.
 \n\\{magit-annex-list-mode-map}"
   :group 'magit-modes
   (hack-dir-local-variables-non-file-buffer))
 
+(defun magit-annex-list-arguments ()
+  (transient-args 'magit-annex-list))
+
 ;;;###autoload
 (defun magit-annex-list-files (&optional args)
   "List annex files.
 \('git annex list [ARGS]')"
   (interactive (magit-annex-list-arguments))
-  (magit-mode-setup #'magit-annex-list-mode nil args))
+  (magit-annex-list-setup-buffer nil args))
 
 ;;;###autoload
 (defun magit-annex-list-dir-files (directory &optional args)
@@ -678,9 +788,14 @@ on the files selected by the region (if active) or the file at point.
                                                    nil nil t)
                               (magit-toplevel)))
          (magit-annex-list-arguments)))
-  (magit-mode-setup #'magit-annex-list-mode directory args))
+  (magit-annex-list-setup-buffer directory args))
 
-(defun magit-annex-list-refresh-buffer (&rest _)
+(defun magit-annex-list-setup-buffer (directory args)
+  (magit-setup-buffer #'magit-annex-list-mode nil
+    (magit-annex-buffer-directory directory)
+    (magit-buffer-arguments args)))
+
+(defun magit-annex-list-refresh-buffer ()
   "Refresh content of a `magit-annex-list-mode' buffer."
   (magit-insert-section (annex-list-buffer)
     (run-hooks 'magit-annex-list-sections-hook)))
@@ -691,14 +806,14 @@ on the files selected by the region (if active) or the file at point.
 
 (defun magit-annex-list-insert-files ()
   "Insert output of `git annex list'."
-  (let* ((subdir (car magit-refresh-args))
+  (let* ((subdir magit-annex-buffer-directory)
          (heading (if subdir
                       (format "Annex files in %s:" subdir)
                     "Annex files:")))
     (magit-insert-section (annex-list-buffer)
       (magit-insert-heading heading)
       (magit-git-wash #'magit-annex-list-wash
-        "annex" "list" magit-refresh-args))))
+        "annex" "list" magit-annex-buffer-directory magit-buffer-arguments))))
 
 (defconst magit-annex-list-line-re "\\([_X]+\\) \\(.*\\)$")
 
